%global bind_uid                  25
%global bind_gid                  25
%global selinuxbooleans           named_write_master_zones=1
%undefine _strict_symbol_defs_build

Summary:  BIND (Berkeley Internet Name Domain), implementation of the Domain Name System (DNS) protocol.
Name:     bind
Version:  9.18.18
Release:  3%{?dist}
License:  MPLv2.0
URL:      https://www.isc.org/bind/
Source0:  https://downloads.isc.org/isc/bind9/%{version}/bind-%{version}.tar.xz
Source1:  named.sysconfig
Source2:  named.logrotate
Source3:  named.conf
Source4:  named.rfc1912.zones
Source5:  named.root.key
Source6:  trusted-key.key
Source8:  named.rwtab
# for named.ca
Source9:  https://www.internic.net/domain/named.root
Source10: named.localhost
Source11: named.loopback
Source12: named.empty
Source13: named.conf.sample
Source14: named.service
Source15: named-setup-rndc.service
Source17: generate-rndc-key.sh
Source19: bind.tmpfiles.d

Patch0001: 0001-CVE-2023-3341.patch
Patch0002: 0002-CVE-2023-4236.patch

BuildRequires:  gcc make autoconf pkgconfig libtool
BuildRequires:  openssl-devel libcap-devel libidn2-devel libxml2-devel libuv-devel
# for DLZ
BuildRequires:  openldap-devel libpq-devel sqlite-devel mariadb-connector-c-devel
BuildRequires:  systemd-rpm-macros python3-ply
BuildRequires:  selinux-policy findutils sed
# configuration, https://gitlab.isc.org/isc-projects/bind9/-/blob/main/doc/arm/build.inc.rst
BuildRequires:  krb5-devel lmdb-devel json-c-devel libmaxminddb-devel
BuildRequires:  fstrm-devel protobuf-c-devel
BuildRequires:  libnghttp2-devel
BuildRequires:  python3-sphinx

%{?systemd_ordering}
Requires:       coreutils
Requires:       bind-libs = %{version}-%{release}
Requires(pre):  shadow-utils
Requires(post): shadow-utils glibc-common grep
Requires(post): ((policycoreutils-python-utils and libselinux-utils) if (selinux-policy-targeted or selinux-policy-mls))
Requires(post): ((selinux-policy and selinux-policy-base) if (selinux-policy-targeted or selinux-policy-mls))


%description
The BIND name server, named, can act as an authoritative name server,
recursive resolver, DNS forwarder, or all three simultaneously.
It implements views for split-horizon DNS, automatic DNSSEC zone signing
and key management, catalog zones to facilitate provisioning of zone data
throughout a name server constellation, response policy zones (RPZ) to
protect clients from malicious data, response rate limiting (RRL) and
recursive query limits to reduce distributed denial of service attacks,
and many other advanced DNS features. BIND also includes a suite of
administrative tools, including the dig and delv DNS lookup tools,
nsupdate for dynamic DNS zone updates, rndc for remote name server
administration, and more.

The BIND 9 Administrator Reference Manual (ARM) can be viewed at
https://bind9.readthedocs.io/en/latest/index.html.


%package libs
Summary:    Libraries for BIND DNS packages
Provides:   bind-libs-lite = %{version}-%{release}

%description libs
Libraries used by named DNS server and utilities in bind-utils package.

%package utils
Summary:    Utilities for querying DNS name servers
Requires:   bind-libs = %{version}-%{release}

%description utils
Utilities for querying DNS name servers to find out information about Internet
hosts, provide you with the IP addresses for given host names, as well as
other information about registered domains and network addresses.


%package dnssec-utils
Summary:    DNSSEC keys and zones management utilities
Requires:   bind-libs = %{version}-%{release}
Recommends: bind-utils

%description dnssec-utils
Utilities for editing DNSSEC keys and BIND zone files provide generation,
revocation and verification of keys and DNSSEC signatures in zone files.
Please install bind-dnssec-utils when signing a DNS zone or maintain keys for it.


%package devel
Summary:    Header files and librariesfor bind-dyndb-ldap
Provides:   bind-lite-devel = %{version}-%{release}
Requires:   bind-libs = %{version}-%{release}
Requires:   openssl-devel libxml2-devel libcap-devel
Requires:   krb5-devel lmdb-devel json-c-devel fstrm-devel protobuf-c-devel libmaxminddb-devel

%description devel
Header files and libraries are required for building bind-dyndb-ldap. Upstream no longer supports
nor recommends bind libraries for third party applications.


%package dlz-filesystem
Summary:    BIND server filesystem DLZ module
Requires:   bind = %{version}-%{release}

%description dlz-filesystem
Dynamic Loadable Zones filesystem module for BIND server.

%package dlz-ldap
Summary:    BIND server ldap DLZ module
Requires:   bind = %{version}-%{release}

%description dlz-ldap
Dynamic Loadable Zones LDAP module for BIND server.

%package dlz-mysql
Summary:    BIND server mysql and mysqldyn DLZ modules
Requires:   bind = %{version}-%{release}
Provides:   %{name}-dlz-mysqldyn = %{version}-%{release}

%description dlz-mysql
Dynamic Loadable Zones MySQL module for BIND server.
Supports mysqldyn module with dynamic DNS updates (DDNS).

%package dlz-sqlite3
Summary:    BIND server sqlite3 DLZ module
Requires:   bind = %{version}-%{release}

%description dlz-sqlite3
Dynamic Loadable Zones sqlite3 module for BIND server.


%prep
%autosetup -p1 -n %{name}-%{version}

%build
export CFLAGS="$CFLAGS $RPM_OPT_FLAGS"
export LDFLAGS="$LDFLAGS -pie -Wl,-z,relro,-z,now,-z,nodlopen,-z,noexecstack"
export STD_CDEFINES="$CPPFLAGS"
autoreconf -fvi
mkdir build
#for dlz build
mkdir -p build/contrib/dlz
cp -frp contrib/dlz/modules build/contrib/dlz/modules

pushd build
%define _configure "../configure"
%configure \
  --includedir=%{_includedir}/bind9 \
  --with-openssl \
  --localstatedir=%{_var} \
  --with-pic \
  --disable-static \
  --with-tuning=large \
  --with-libidn2 \
  --with-maxminddb \
  --with-gssapi=yes \
  --with-lmdb=yes \
  --with-json-c \
  --enable-dnstap \
  --enable-fixed-rrset \
  --enable-full-report

pushd lib
SRCLIB="../../../lib"
(cd dns && ln -s ${SRCLIB}/dns/dnstap.proto)
popd

%make_build

pushd contrib/dlz/modules
for d in filesystem ldap mysql mysqldyn sqlite3; do
  [ -e $d/Makefile ] && make -C $d CFLAGS="-fPIC -I../include $CFLAGS $LDFLAGS"
done
popd

popd # build


%install
mkdir -p ${RPM_BUILD_ROOT}%{_sysconfdir}/logrotate.d
install -D -m 644 %{SOURCE1} ${RPM_BUILD_ROOT}%{_sysconfdir}/sysconfig/named
install -D -m 644 %{SOURCE2} ${RPM_BUILD_ROOT}/etc/logrotate.d/named
install -D -m 640 %{SOURCE3} %{SOURCE4} %{SOURCE5} %{SOURCE6} ${RPM_BUILD_ROOT}%{_sysconfdir}/
touch ${RPM_BUILD_ROOT}%{_sysconfdir}/rndc.{key,conf}
mkdir -p ${RPM_BUILD_ROOT}%{_sysconfdir}/named
mkdir -p ${RPM_BUILD_ROOT}%{_sysconfdir}/rwtab.d
install -m 644 %{SOURCE8} ${RPM_BUILD_ROOT}%{_sysconfdir}/rwtab.d/named

mkdir -p ${RPM_BUILD_ROOT}%{_libdir}/{bind,named}

mkdir -p ${RPM_BUILD_ROOT}%{_localstatedir}/named/{slaves,data,dynamic}
install -m 640 %{SOURCE9} ${RPM_BUILD_ROOT}%{_localstatedir}/named/named.ca
install -m 640 %{SOURCE10} %{SOURCE11} %{SOURCE12} ${RPM_BUILD_ROOT}%{_localstatedir}/named/
mkdir -p ${RPM_BUILD_ROOT}%{_localstatedir}/log
# Ghost config files:
touch ${RPM_BUILD_ROOT}%{_localstatedir}/log/named.log

mkdir -p ${RPM_BUILD_ROOT}%{_mandir}/{man1,man5,man8}
pushd ${RPM_BUILD_ROOT}%{_mandir}/man8
ln -s ddns-confgen.8.gz tsig-keygen.8.gz
popd

mkdir -p ${RPM_BUILD_ROOT}/run/named

mkdir -p ${RPM_BUILD_ROOT}%{_unitdir}
install -D -m 644 %{SOURCE14} %{SOURCE15} ${RPM_BUILD_ROOT}%{_unitdir}/

mkdir -p ${RPM_BUILD_ROOT}%{_libexecdir}
install -D -m 755 %{SOURCE17} ${RPM_BUILD_ROOT}%{_libexecdir}/

mkdir -p ${RPM_BUILD_ROOT}%{_includedir}/bind9/isc
install -m 644 lib/isc/errno2result.h ${RPM_BUILD_ROOT}%{_includedir}/bind9/isc

mkdir -p ${RPM_BUILD_ROOT}%{_tmpfilesdir}
install -D -m 644 %{SOURCE19} ${RPM_BUILD_ROOT}%{_tmpfilesdir}/named.conf

pushd build
%make_install
  pushd contrib/dlz/modules
  for DIR in filesystem ldap mysql mysqldyn sqlite3; do
    %make_install -C $DIR libdir=%{_libdir}/named
  done
  pushd ${RPM_BUILD_ROOT}/%{_libdir}/bind
    cp -s ../named/dlz_*.so .
  popd
  mkdir -p doc/{mysql,mysqldyn}
  cp -p mysqldyn/testing/README doc/mysqldyn/README.testing
  cp -p mysqldyn/testing/* doc/mysqldyn
  cp -p mysql/testing/* doc/mysql
  popd
popd

find ${RPM_BUILD_ROOT}%{_libdir} -name "*.la" -exec rm -f {} \;
rm -f ${RPM_BUILD_ROOT}/etc/bind.keys

# sample bind configuration files for %%doc:
mkdir -p sample/etc sample/var/named/{data,slaves}
install -m 644 %{SOURCE13} sample/etc/named.conf
install -m 644 %{SOURCE3} named.conf.default
install -m 644 %{SOURCE8} sample/etc/named.rfc1912.zones
install -m 644 %{SOURCE5} %{SOURCE6} sample/var/named/
install -m 644 %{SOURCE9} sample/var/named/named.ca
for f in my.internal.zone.db slaves/my.slave.internal.zone.db slaves/my.ddns.internal.zone.db my.external.zone.db; do 
  echo '@ in soa localhost. root 1 3H 15M 1W 1D
  ns localhost.' > sample/var/named/$f
done
:


%pre
if [ "$1" -eq 1 ]; then
  /usr/sbin/groupadd -g %{bind_gid} -f -r named >/dev/null 2>&1 || :
  /usr/sbin/useradd  -u %{bind_uid} -r -N -M -g named -s /sbin/nologin -d /var/named -c Named named >/dev/null 2>&1 || :
fi
:

%post
/sbin/ldconfig
if [ -e "%{_sysconfdir}/selinux/config" ]; then
  %selinux_set_booleans -s targeted %{selinuxbooleans}
  %selinux_set_booleans -s mls %{selinuxbooleans}
fi
if [ "$1" -eq 1 ]; then
  # Initial installation
  [ -x /sbin/restorecon ] && /sbin/restorecon /etc/rndc.* /etc/named.* >/dev/null 2>&1
  [ -e /etc/rndc.key ] && chown root:named /etc/rndc.key
  [ -e /etc/rndc.key ] && chmod 0640 /etc/rndc.key
else
  # Upgrade
  if getent passwd named | grep ':/bin/false$' >/dev/null; then
    /sbin/usermod -s /sbin/nologin named
  fi
  if /usr/bin/named-checkconf -p /etc/named.conf 2>/dev/null | grep -q named.iscdlv.key; then
      sed -e -i 's/named\.iscdlv\.key/named.root.key/' /etc/named.conf || :
  fi
fi
%systemd_post named.service
:

%preun
# Package removal
%systemd_preun named.service

%postun
/sbin/ldconfig
# Package upgrade
%systemd_postun_with_restart named.service
if [ -e "%{_sysconfdir}/selinux/config" ]; then
  %selinux_unset_booleans -s targeted %{selinuxbooleans}
  %selinux_unset_booleans -s mls %{selinuxbooleans}
fi


%files
%doc CHANGES NEWS README.md named.conf.default 
%license COPYRIGHT COPYING LICENSE
%dir %{_libdir}/bind
%{_libdir}/bind/filter-aaaa.so
%{_libdir}/bind/filter-a.so
%dir %{_libdir}/named
%{_libdir}/named/*.so
%exclude %{_libdir}/named/dlz_*.so
%config(noreplace) %verify(not md5 size mtime) %{_sysconfdir}/sysconfig/named
%config(noreplace) %attr(0644,root,named) %{_sysconfdir}/named.root.key
%config(noreplace) %{_sysconfdir}/logrotate.d/named
%{_sysconfdir}/rwtab.d/named
%{_tmpfilesdir}/named.conf
%{_unitdir}/named.service
%{_unitdir}/named-setup-rndc.service
%{_bindir}/named-journalprint
%{_bindir}/named-checkconf
%{_bindir}/named-rrchecker
%{_bindir}/mdig
%{_sbindir}/named
%{_sbindir}/rndc*
%{_libexecdir}/generate-rndc-key.sh
%{_mandir}/man1/named-journalprint.1*
%{_mandir}/man1/named-checkconf.1*
%{_mandir}/man1/named-rrchecker.1*
%{_mandir}/man1/mdig.1*
%{_mandir}/man5/named.conf.5*
%{_mandir}/man8/named.8*
%{_mandir}/man5/rndc.conf.5*
%{_mandir}/man8/rndc.8*
%{_mandir}/man8/rndc-confgen.8*
%{_mandir}/man8/filter-aaaa.8.gz
%{_mandir}/man8/filter-a.8.gz
%doc sample/

%defattr(0640,root,named,0750)
%dir %{_sysconfdir}/named
%config(noreplace) %verify(not link) %{_sysconfdir}/named.conf
%config(noreplace) %verify(not link) %{_sysconfdir}/named.rfc1912.zones

%defattr(0660,root,named,01770)
%dir %{_localstatedir}/named

%defattr(0660,named,named,0770)
%dir %{_localstatedir}/named/slaves
%dir %{_localstatedir}/named/data
%dir %{_localstatedir}/named/dynamic
%ghost %{_localstatedir}/log/named.log

%defattr(0640,root,named,0750)
%config %verify(not link) %{_localstatedir}/named/named.ca
%config %verify(not link) %{_localstatedir}/named/named.localhost
%config %verify(not link) %{_localstatedir}/named/named.loopback
%config %verify(not link) %{_localstatedir}/named/named.empty
%ghost %config(noreplace) %{_sysconfdir}/rndc.key
%ghost %config(noreplace) %{_sysconfdir}/rndc.conf

%defattr(-,named,named,-)
%dir /run/named

%files libs
%license COPYRIGHT LICENSE COPYING
%{_libdir}/libbind9-%{version}*.so
%{_libdir}/libisccc-%{version}*.so
%{_libdir}/libns-%{version}*.so
%{_libdir}/libdns-%{version}*.so
%{_libdir}/libirs-%{version}*.so
%{_libdir}/libisc-%{version}*.so
%{_libdir}/libisccfg-%{version}*.so


%files utils
%{_sysconfdir}/trusted-key.key
%{_bindir}/dig
%{_bindir}/delv
%{_bindir}/host
%{_bindir}/nslookup
%{_bindir}/nsupdate
%{_bindir}/arpaname
%{_bindir}/dnstap-read
%{_bindir}/nsec3hash
%{_bindir}/named-checkzone
%{_bindir}/named-nzd2nzf
%{_bindir}/named-compilezone
%{_sbindir}/ddns-confgen
%{_sbindir}/tsig-keygen
%{_mandir}/man1/dig.1*
%{_mandir}/man1/delv.1*
%{_mandir}/man1/host.1*
%{_mandir}/man1/nslookup.1*
%{_mandir}/man1/nsupdate.1*
%{_mandir}/man1/arpaname.1*
%{_mandir}/man1/dnstap-read.1*
%{_mandir}/man1/nsec3hash.1*
%{_mandir}/man1/named-checkzone.1*
%{_mandir}/man1/named-nzd2nzf.1*
%{_mandir}/man1/named-compilezone.1*
%{_mandir}/man8/ddns-confgen.8*
%{_mandir}/man8/tsig-keygen.8*

%files dnssec-utils
%{_bindir}/dnssec*
%{_mandir}/man1/dnssec*.1*

%files devel
%{_libdir}/libbind9.so
%{_libdir}/libisccc.so
%{_libdir}/libns.so
%{_libdir}/libdns.so
%{_libdir}/libirs.so
%{_libdir}/libisc.so
%{_libdir}/libisccfg.so
%dir %{_includedir}/bind9
%{_includedir}/bind9/bind9
%{_includedir}/bind9/isccc
%{_includedir}/bind9/ns
%{_includedir}/bind9/dns
%{_includedir}/bind9/dst
%{_includedir}/bind9/irs
%{_includedir}/bind9/isc
%{_includedir}/bind9/isccfg


%files dlz-filesystem
%{_libdir}/{named,bind}/dlz_filesystem_dynamic.so

%files dlz-mysql
%{_libdir}/{named,bind}/dlz_mysql_dynamic.so
%{_libdir}/{named,bind}/dlz_mysqldyn_mod.so
%doc build/contrib/dlz/modules/doc/mysql
%doc build/contrib/dlz/modules/doc/mysqldyn

%files dlz-ldap
%{_libdir}/{named,bind}/dlz_ldap_dynamic.so
%doc contrib/dlz/modules/ldap/testing/*

%files dlz-sqlite3
%{_libdir}/{named,bind}/dlz_sqlite3_dynamic.so
%doc contrib/dlz/modules/sqlite3/testing/*


%changelog
* Fri Nov 03 2023 Xin Cheng <denisecheng@tencent.com> - 9.18.18-3
- Add 0001-CVE-2023-3341.patch
- Add 0002-CVE-2023-4236.patch

* Fri Sep 08 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 9.18.18-2
- Rebuilt for OpenCloudOS Stream 23.09

* Tue Aug 22 2023 kianli <kianli@tencent.com> - 9.18.18-1
- Upgrade to 9.18.18

* Fri Aug 18 2023 Wang Guodong <gordonwwang@tencent.com> - 9.18.2-5
- Rebuilt for mariadb-connector-c 3.3.5

* Thu Aug 3 2023 Shuo Wang <abushwang@tencent.com> - 9.18.2-4
- Rebuilt for libpq 15.3

* Fri Apr 28 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 9.18.2-3
- Rebuilt for OpenCloudOS Stream 23.05

* Fri Mar 31 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 9.18.2-2
- Rebuilt for OpenCloudOS Stream 23

* Thu May 19 2022 rockerzhu <rockerzhu@tencent.com> - 9.18.2-1
- Initail build
